/************************************************************************************************
 All ADuC706x library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.
 Author        : ADI - Apps            www.analog.com/AduC7060

 Date          : May 2008

 File          : Calibrate.c

 Hardware      : ADuC706x

 Description   : This file contains routines used to calibrate an RTD.
*************************************************************************************************/

#include <Analogdevices\ioaduc7060.h >
#include <math.h>
#include <intrinsics.h>
#include <stdio.h>
#include <string.h>
#include <C:\ADuC7XXXV0.1\ADuC706X\code\IAR\RTD_PieceWise_Linearization\RTDdefs.h>

float fADC0cal = 0.0;
unsigned char szIn[8];
extern float fADC0norm;
extern float fscale0;
extern float foffset0;
extern float fRprevcal;
extern unsigned char szTemp[64];

signed char Cal (float Rcal);
extern float T_rtd (float r);
extern float R_rtd (float r);
extern float Tmin_rtd (void);
extern float Tmax_rtd (void);
extern float Rmin_rtd (void);
extern float Rmax_rtd (void);
extern unsigned char bSerialIn;
extern volatile unsigned char ucUserEntry;

extern void PrintString (void);
float ParseInputR(void);
float ParseInputT(void);

void Calibrate (void);

void Calibrate ()
{
  unsigned char i = 0;
  signed char result;                 // calibration result indicator
  float Rcal, Tcal;                   // calibration resistance & temperature
  float Rrtd, Trtd;                   // current resistance & temperature of RTD

  // determine RTD temperature based on ADC result & current cal coef
  fADC0cal = fADC0norm;               // capture current ADC0 reading
  Rrtd = (fADC0cal*fscale0)+foffset0;   // current RTD resistance [ohms]
  Trtd = T_rtd(Rrtd);                 // current RTD temperature [degC]

  // display calibration menu
  sprintf ( (char*)szTemp, "\r\n\r\n");
  PrintString();

  sprintf ( (char*)szTemp,"most recent reading: %+8.2fC, %+8.2fohms\r\n", Trtd, Rrtd);
  PrintString();

  sprintf ( (char*)szTemp,"CALIBRATION MENU....\r\n");
  PrintString();

  sprintf ( (char*)szTemp,"  T: enter known RTD temperature at most recent reading (INCL. - or + sign\r\n");
  PrintString();

  sprintf ( (char*)szTemp,"  R: enter known RTD resistance at most recent reading\r\n");
  PrintString();

  sprintf ( (char*)szTemp,"  D: restore default calibration coefficients\r\n");
  PrintString();

  sprintf ( (char*)szTemp,"  any other key: cancel calibration & resume normal operation\r\n");
  PrintString();

  sprintf ( (char*)szTemp,"select option: ");
  PrintString();
  while (bSerialIn == 0)
  {}
    // restore serial_in to zero, ready for next serial interrupt
  bSerialIn = 0;
  // get keystroke & perform corresponding action
  switch (ucUserEntry)
 {
    case 't' :
    case 'T' :
               COMTX = 0x54;
               sprintf ( (char*)szTemp,"enter RTD temperature [degC]: \r\n\r");
               PrintString();

               sprintf ( (char*)szTemp,"For example, +021.4 ");
               PrintString();
               for (i=0;i<6;i++)
               {
                   while (bSerialIn == 0)
                   {}
                   szIn[i] = ucUserEntry;
                   bSerialIn = 0;
               }
               Tcal = ParseInputT();
               sprintf ( (char*)szTemp,"Temperature entered, %+8.2fC\r\n", Tcal );
               PrintString();
               if (Tcal == 0.0)
               {
                  result = -2;
               }
               else
               {
                  result = 0;
                  Rcal=R_rtd(Tcal);
                  result=Cal(Rcal);
               }
               break;
    case 'r' :
    case 'R' :
               COMTX = 0x52;
               sprintf ( (char*)szTemp, "R\r\n");
               PrintString();
               sprintf ( (char*)szTemp, "enter RTD resistance [ohms]: R\r\n");
               PrintString();
               sprintf ( (char*)szTemp,"For example, 1250.0 R\r\n");
               PrintString();
               for (i=0;i<6;i++)
               {
                   while (bSerialIn == 0)
                   {}
                   szIn[i] = ucUserEntry;
                   bSerialIn = 0;
               }
               Rcal = ParseInputR();
               sprintf ( (char*)szTemp, "Resistance entered, %+8.2fohms\r\n", Rcal );
               PrintString();
               if (Rcal == 0.0)
                  result = -2;
               Tcal=T_rtd(Rcal);
               result=Cal(Rcal);
               result=0;
               break;
    case 'd' :
    case 'D' : printf ("D\r\n");
               fscale0 = DEFAULTSCALE0;
               foffset0 = DEFAULTOFFSET0;
               fRprevcal = 0;
               result = -1;
               break;
    default  : result = -2;
               break;
  }

  // indicate results
  switch (result)
  {
    case -2 :
      break;
    case -1 :
      sprintf ( (char*)szTemp, "\r\n");
      PrintString();
      sprintf ( (char*)szTemp, "RESULT: Default Cal Coefficients Restored");
      PrintString();
      break;
    case 0 :
      sprintf ( (char*)szTemp, "\r\n");
      PrintString();
      sprintf ( (char*)szTemp, "RESULT: Calibrated at %fdegC",Tcal);
      PrintString();
      break;
    case 1 :
      sprintf ( (char*)szTemp, "\r\n");
      PrintString();
      sprintf ( (char*)szTemp, "RESULT: Out of Range Input, No Cal Performed");
      PrintString();
      break;
    case 2 :
      sprintf ( (char*)szTemp, "\r\n");
      PrintString();
      sprintf ( (char*)szTemp, "RESULT: Conflict with Previous Cal, No Cal Performed");
      PrintString();
      break;
    default :
      sprintf ( (char*)szTemp, "\r\n");
      PrintString();
      sprintf ( (char*)szTemp, "RESULT: Unknown Calibration Error");
      PrintString();
      break;
  }

}
/******************************************************************************
// Raw Calibration Suboutine
//    Input: Rcal = expected RTD resistance at current temperature
//    Output: Cal() = error indication (per comments below)
//    Globals: fscale0, foffset0, fRprevcal, fADC0cal
//    Function: Performs a calibration at current data point while preserving
// cal performed at previous cal point (per "fRprevcal" globals).
********************************************************************************/
signed char Cal (float Rcal)
{
  signed char returnerr;  // to indicate error (0 = no error)
  float fADC0prevcal;      // ADC result at previous cal point
  unsigned char ucFirstCal;

  returnerr=3;  // default error code (indicates program fault)
  ucFirstCal = 0;
  // determine ADC value at previous cal point (from Rprevcal)
  fADC0prevcal = (fRprevcal-foffset0)/fscale0;
  if (fADC0prevcal == 0)
      ucFirstCal = 1;
  // first check to see if input is out of range or conflicting
  if (Rcal<Rmin_rtd() || Rcal>Rmax_rtd())
    returnerr=1;  // indicate out-of-range Rcal input
  else if ((Rcal>=fRprevcal && fADC0cal<=fADC0prevcal) && (ucFirstCal != 1))
    returnerr=2;  // indicate conflict with previous cal
  else if ((Rcal<=fRprevcal && fADC0cal>=fADC0prevcal) && (ucFirstCal != 1))
    returnerr=2;  // indicate conflict with previous cal

  // if everything looks good, go ahead and perform cal
  else
  {
      if (ucFirstCal == 1) // No previous calibration point
      {
           fscale0=(Rcal)/(fADC0cal);
          // determine offset using new scale0 value & one known point
          foffset0=Rcal-(fADC0cal*fscale0);
      }
      else
      {
          // scale0 is essentially "rise-over-run", or deltaR/deltaADC
          fscale0=(Rcal-fRprevcal)/(fADC0cal-fADC0prevcal);
          // determine offset using new scale0 value & one known point
          foffset0=Rcal-(fADC0cal*fscale0);
      }
    fRprevcal=Rcal; // we'll need Rprevcal for the next cal point
    returnerr=0;   // indicate succesful calibration
  }

  return (returnerr);
}

float ParseInputT(void)
{
    unsigned char i, ucSignEntry, ucDotEntry, ucNegEntry, ucPosEntry = 0;
    float fEntry, fEntry1, fEntry2, fRemainder = 0.0;
    unsigned ucLoopCnt = 0;

    for ( i = 0 ; i < 7 ; i++ )
    {
        if (szIn[i] == 0x2b)        // check for +
        {
          ucSignEntry = i;
          ucPosEntry = 1;
        }
        if (szIn[i] == 0x2D)        // check for -
        {
          ucSignEntry = i;
          ucNegEntry = 1;
        }
        if (szIn[i] == 0x2E)        // check for .
        {
          ucDotEntry = i;
        }
    }
    if (ucPosEntry == ucNegEntry)
    {
        ucSignEntry = 0;
        ucLoopCnt = ucDotEntry - ucSignEntry;
    }
    else
    {
        ucLoopCnt = ucDotEntry - ucSignEntry;
        ucLoopCnt--;
    }
    fEntry = 0.0;
    i = ucSignEntry;
    if (i < ucDotEntry)
    {
        for (i = ucSignEntry+1; i < ucDotEntry; i++)
        {
            fEntry1 = ((szIn[i]) - 0x30);
            fEntry2 = (fEntry1 * pow(10,ucLoopCnt-1));
            ucLoopCnt--;
            fEntry += fEntry2;
        }
        fRemainder = (((szIn[ucDotEntry+1]) - 0x30)*0.1);
        fEntry += fRemainder;
    }
    else
    {
        return 0.0;
    }
    if (ucNegEntry == 1)
      fEntry = -fEntry;

    return fEntry;
}
float ParseInputR(void)
{
    unsigned char i, ucDotEntry = 0;
    float fEntry, fEntry1, fEntry2, fRemainder = 0.0;
    unsigned ucLoopCnt = 0;

    for ( i = 0 ; i < 7 ; i++ )
    {
        if (szIn[i] == 0x2E)        // check for .
        {
          ucDotEntry = i;
        }
    }
    fEntry = 0.0;
    ucLoopCnt = ucDotEntry - 1;
    for (i = 0; i < ucDotEntry; i++)
    {
        fEntry1 = ((szIn[i]) - 0x30);
        fEntry2 = (fEntry1 * pow(10,ucLoopCnt));
        ucLoopCnt--;
        fEntry += fEntry2;
    }
    fRemainder = (((szIn[ucDotEntry+1]) - 0x30)*0.1);
    fEntry += fRemainder;
    return fEntry;
}